//-----------------------------------------------------
//File: lcd_spi.c
//Auth: ELECTRONIC ASSEMBLY JM
//DATE: 08-07-2012
//-----------------------------------------------------

//---Includes---
#include "sfr_r825.h"
#include "define.h"
#include "lcd_spi.h"

#include "timer.h"

//--- module global varibles ---


//--- waitstates for testing ---
//use these macros to slow the communication down
#define WAITST 0
#define Wait(x)
//#define Wait(x) asm("nop")
/*void Wait (unsigned char waitstates)
{
	unsigned char i=0;
	for (i=0; i<waitstates; i++)
	{
		asm("nop");
	}
}
*/

//-----------------------------------------------------
//Func: initDispl()
//Desc: inits Display
//-----------------------------------------------------
void initDispl(void)
{
	unsigned char i;
	
//init SPI, connected to UART0, define as synchonous interface
	//Port init (dd -> datadirection register; 1 output, 0 input)
	ddCS = 1;
	CS = 1;	
	ddSID = 1;
	SID = 1;
	ddSOD = 0;
	ddSCLK = 1;
	SCLK = 1;
	ddRES = 1;
	RES = 1;
	
	RES = 0;
	Delay(1);
	RES=1;
	Delay(1);

	//SPI mode: Pol 0 (falling edge), UFORM = 0 (LSB first)
	u0mr = 0x01; //synchronous serial mode, internal clock, 1 stop bit, no parity
	u0c0 = 0x00; //f1, CMOS output, falling edge, LSB first
	u0c1 = 0x67; //enable trans, enable rec, enable cont rec
//	u0brg = 0x81; //baud: 1200 u0brg = (f/(BitRate x 16)-1; f= 20MHz/8 -->u0c0 f8
	u0brg = 0x0A; //baud: 115200 u0brg = (f/(BitRate x 16)-1; f= 20MHz/1 -->u0c0 f1

//init Display
		
#ifdef DIP203_4
	WriteIns(0x34);	//8-Bit, RE=1
	WriteIns(0x09);	//4 line mode
	WriteIns(0x30);	//8-Bit, RE=0
	WriteIns(0x0F);	//Display on, cursor on, cursor blink
	WriteIns(0x01);	//clear display
	WriteIns(0x06);	//return home
#endif


#ifdef DIP203_6	
	WriteIns(0x30);	//8-Bit, RE=0
	WriteIns(0x06);	//cursor auto increment
	WriteIns(0x36);	//8-Bit, RE=1, BE=1
	WriteIns(0x09);	//4 line mode

	WriteIns(0x40);
	for (i=0; i<16; i++)
		WriteData(0x00);

	WriteIns(0x30);	//8-Bit, RE=0
	WriteIns(0x0F);	//Display on, cursor on, cursor blink
	WriteIns(0x01);	//clear display
	WriteIns(0x06);	//return home
#endif

}

//-----------------------------------------------------
//Func: WriteChar(character)
//Desc: sends a single character to display
//-----------------------------------------------------
void WriteChar (char character)
{
	WriteData(character);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void WriteString(_far char * string)
{
	do
	{
		WriteData(*string++);
	}
	while(*string);
}

//-----------------------------------------------------
//Func: SetSymb(addresse, data)
//Desc: shows a symbol, if databit 8/6 clear/set all (only applicable with DIP203-6)
//-----------------------------------------------------
void SetSymb (char add, char data)
{
	unsigned char i=0;
	unsigned char cursorpos = GETCURSORADDR();
	WriteIns(0x36);
	WriteIns(add);
	if(data&0x80)
	{
		for (i=0; i<16; i++)
			WriteData(0x00);
	}
	else if(data&0x20)
	{
		data&=0x5F;
		for (i=0; i<16; i++)
			WriteData(data);
	}
	else
		WriteData(data);
	WriteIns(0x30);
	SetPostion(cursorpos);

}

//-----------------------------------------------------
//Func: SetPostion(postion)
//Desc: set postion
//-----------------------------------------------------
void SetPostion(char pos)
{
	WriteIns(LCD_HOME_L1+pos);
}

//-----------------------------------------------------
//Func: DisplayOnOff(control)
//Desc: use definitions of header file to set display
//-----------------------------------------------------
void DisplayOnOff(char data)
{
	WriteIns(0x08+data);
}

//-----------------------------------------------------
//Func: DefineCharacter(memory postion, character data)
//Desc: stores an own defined character
//-----------------------------------------------------
void DefineCharacter(unsigned char postion, unsigned char *data)
{
	unsigned char i=0;
	WriteIns(0x40+8*postion);

	for (i=0; i<8; i++)
	{
		WriteData(data[i]);
	}
	SetPostion(LINE1);
}
//-----------------------------------------------------
//Func: ClrDisplay
//Desc: Clears entire Display content and set home pos
//-----------------------------------------------------
void ClrDisplay(void)
{
	WriteIns(0x01);
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: WriteIns(instruction)
//Desc: sends instruction to display
//-----------------------------------------------------
static void WriteIns(char ins)
{
	CheckBusy();
	CS = 0;
	SPI_put(0x1F);
	SPI_put(ins & 0x0F);
	SPI_put((ins>>4) & 0x0F);
	CS = 1;
}

//-----------------------------------------------------
//Func: WriteData(data)
//Desc: sends data to display
//-----------------------------------------------------
static void WriteData(char data)
{
	CheckBusy();
	CS = 0;
	SPI_put(0x5F);
	SPI_put(data & 0x0F);
	SPI_put((data>>4) & 0x0F);
	CS = 1;
}

//-----------------------------------------------------
//Func: CheckBusy()
//Desc: checks if display is idle
//-----------------------------------------------------
unsigned char CheckBusy(void)
{
	unsigned char readData = 1;

	do
	{
		CS = 0;
		SPI_put(0x3F);
		SPI_put(0x00);
		while(ri_u0c1 == 0);
		readData= ~u0rbl;
		CS = 1;
	}while(readData&0x80); //check for busyflag
	
	return readData;
}

//-----------------------------------------------------
//Func: SPI_put()
//Desc: waits till data is sent over the spi interface
//-----------------------------------------------------
static void SPI_put (unsigned char byte)
{
	u0tb = ~byte;
	while(ti_u0c1 == 0);
	while(txept_u0c0 == 0);
}